@@ -9,27 +9,28 @@ require Exporter;
 use constant EULER => 2.718281828459045235360287;
 use constant PI    => 3.141592653589793238462643;
 
-use constant B_SUM          => 0;
-use constant B_DIFFERENCE   => 1;
-use constant B_PRODUCT      => 2;
-use constant B_DIVISION     => 3;
-use constant U_MINUS        => 4;
-use constant U_P_DERIVATIVE => 5;
-use constant U_T_DERIVATIVE => 6;
-use constant B_EXP          => 7;
-use constant B_LOG          => 8;
-use constant U_SINE         => 9;
-use constant U_COSINE       => 10;
-use constant U_TANGENT      => 11;
-use constant U_COTANGENT    => 12;
-use constant U_ARCSINE      => 13;
-use constant U_ARCCOSINE    => 14;
-use constant U_ARCTANGENT   => 15;
-use constant U_ARCCOTANGENT => 16;
-use constant U_SINE_H       => 17;
-use constant U_COSINE_H     => 18;
-use constant U_AREASINE_H   => 19;
-use constant U_AREACOSINE_H => 20;
+use constant B_SUM            => 0;
+use constant B_DIFFERENCE     => 1;
+use constant B_PRODUCT        => 2;
+use constant B_DIVISION       => 3;
+use constant U_MINUS          => 4;
+use constant U_P_DERIVATIVE   => 5;
+use constant U_T_DERIVATIVE   => 6;
+use constant B_EXP            => 7;
+use constant B_LOG            => 8;
+use constant U_SINE           => 9;
+use constant U_COSINE         => 10;
+use constant U_TANGENT        => 11;
+use constant U_COTANGENT      => 12;
+use constant U_ARCSINE        => 13;
+use constant U_ARCCOSINE      => 14;
+use constant U_ARCTANGENT     => 15;
+use constant U_ARCCOTANGENT   => 16;
+use constant U_SINE_H         => 17;
+use constant U_COSINE_H       => 18;
+use constant U_AREASINE_H     => 19;
+use constant U_AREACOSINE_H   => 20;
+use constant B_ARCTANGENT_TWO => 21;
 
 use constant T_OPERATOR => 0;
 use constant T_CONSTANT => 1;
@@ -64,6 +65,7 @@ our %EXPORT_TAGS = (
           U_COSINE_H
           U_AREASINE_H
           U_AREACOSINE_H
+          B_ARCTANGENT_TWO
 
           T_OPERATOR
           T_CONSTANT
@@ -76,7 +78,7 @@ our @EXPORT_OK = ( @{ $EXPORT_TAGS{'all'} } );
 
 our @EXPORT = qw(
 );
-our $VERSION = '0.508';
+our $VERSION = '0.509';
 
 1;
 __END__
